-- ****************************************************************************************************
-- Change Log:
-- Version 3.2.221117
--
-- 2022-11-17
-- Updated how the "Add Line" checkbox saves it's value to the variable.
--
-- 2022-11-15
-- Updated interface number for 10.0.2
-- Added support for Evoker
--
-- 2022-10-26
-- Updated interface number for 10.0.0
--
-- 2022-06-07
-- Updated interface number for 9.2.5 (sorry for never updating to 9.2.0 - life got the better of me
-- for a while there...)
--
-- 2021-11-02
-- Update interface number for 9.1.5
--
-- 2021-07-02
-- Updated version and interface numbers for 9.1
--
-- 2021-03-11
-- Updated version and interface numbers for 9.0.5
--
-- 2020-12-30
-- Updated the variables load logic to be based on the VARIABLES_LOADED event
-- Corrected tooltip on Interface > Addons page to show better explanation of the feature
--
-- 2020-12-29
-- Added Interface > Addons page for Khes'yc iLvl with toggle to add a blank line before
-- details added by Khes'yc iLvl
--
-- 2020-11-26
-- Updated KhesyciLvlGetClassColor to use classId instead of className to work in different
-- language versions of the game
--
-- 2020-11-18
-- Updated interface build number for 9.0.2
--
-- 2020-09-27
-- Initial build. Scans other players for spec and iLvl.
-- Limited distribution for personal use only.
-- ****************************************************************************************************

--declarations
local addon = ...;
local addonTitle = GetAddOnMetadata(addon, "Title");
local addonVersion = GetAddOnMetadata(addon, "Version");
local isWaiting = false;
local name = UnitName("player");
local scanName = UnitName("player");
local realm = GetRealmName();
local scanRealm = GetRealmName();
local class = "";
local spec = "";
local iLvl = 0;

--GLOBALS
local _G = _G;
local InterfaceOptions_AddCategory = _G.InterfaceOptions_AddCategory;

--Create the Blizzard addon option frame
local panel = CreateFrame("Frame", addonTitle .. "BlizzOptions");
panel.name = addonTitle;
InterfaceOptions_AddCategory(panel);

local fs = panel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
fs:SetPoint("TOPLEFT", 10, -15);
fs:SetJustifyH("LEFT");
fs:SetJustifyV("TOP");
fs:SetText(addonTitle);

local st = panel:CreateFontString(nil, "OVERLAY", "SystemFont_Outline");
st:SetPoint("TOPLEFT", 10, -48);
st:SetPoint("BOTTOMRIGHT", panel, "TOPRIGHT", 10, -80);
st:SetJustifyH("LEFT");
st:SetJustifyV("TOP");
st:SetText("Please consider supporting this and our other projects at www.khesyc.com via Patreon at www.patreon.com/khesyc or Venmo at @Khesyc.");

--review how to add interface option with check box
local chkAddLine = CreateFrame("CheckButton", "ki_chkAddLine", panel, "ChatConfigCheckButtonTemplate");
chkAddLine:SetPoint("TOPLEFT", 10, -85);
chkAddLine.tooltip = "Add a blank line before Khes'yc iLvl details?";
chkAddLine:SetScript("OnClick", function(self)
	ki_addLine = chkAddLine:GetChecked();
end)

local lblAddLine = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
lblAddLine:SetPoint("TOPLEFT", 35, -90);
lblAddLine:SetJustifyH("LEFT");
lblAddLine:SetJustifyV("TOP");
lblAddLine:SetText("Add Blank Line?");

--Create event handler frame
local ki_Event = CreateFrame("frame", "ki_Event");
ki_Event:RegisterEvent("VARIABLES_LOADED");
ki_Event:RegisterEvent("UPDATE_MOUSEOVER_UNIT");

ki_Event:SetScript("OnEvent", function(self, event, ...)
	if (event == "VARIABLES_LOADED") then
		KhesyciLvlInit();
	elseif (event == "UPDATE_MOUSEOVER_UNIT") then
		if not isWaiting then
			KhesyciLvlCheckName();
		end
	elseif (event == "INSPECT_READY") then
		KhesyciLvlInspect(...);
		ki_Event:UnregisterEvent("INSPECT_READY");
	end
end);

function KhesyciLvlInit()
	if (ki_addLine == nil) then ki_addLine = true end

	chkAddLine:SetChecked(ki_addLine);
end

function KhesyciLvlGetClassColor(classId)
	if (classId == 1) then --Warrior
		return 0.78, 0.61, 0.43;
	elseif (classId == 3) then --Hunter
		return 0.67, 0.83, 0.45;
	elseif (classId == 8) then --Mage
		return 0.25, 0.78, 00.92;
	elseif (classId == 4) then --Rogue
		return 1.00, 0.96, 0.41;
	elseif (classId == 5) then --Priest
		return 1.00, 1.00, 1.00;
	elseif (classId == 9) then --Warlock
		return 0.53, 0.53, 0.93;
	elseif (classId == 2) then --Paladin
		return 0.96, 0.55, 0.73;
	elseif (classId == 11) then --Druid
		return 1.00, 0.49, 0.04;
	elseif (classId == 7) then --Shaman
		return 0.00, 0.44, 0.87;
	elseif (classId == 10) then --Monk
		return 0.00, 1.00, 0.59;
	elseif (classId == 12) then --Demon Hunter
		return 0.64, 0.19, 0.79;
	elseif (classId == 6) then --Death Knight
		return 0.77, 0.12, 0.23;
	elseif (classId == 13) then --Evoker
		return 0.20, 0.58, 0.50;
	end
end

function KhesyciLvlSetWaiting()
	isWaiting = true;
	C_Timer.After(0.5, function() KhesyciLvlClearWaiting() end);
end

function KhesyciLvlClearWaiting()
	isWaiting = false;
	KhesyciLvlCheckName();
end

function KhesyciLvlCheckName()
	if (UnitIsPlayer("mouseover") and isWaiting == false) then
		local newName, newRealm = UnitName("mouseover");
		if (newRealm == nil or newRealm == '') then
			newRealm = GetRealmName();
		end

		if (newName ~= scanName or newRealm ~= scanRealm) then
			scanName = newName;
			scanRealm = newRealm;

			KhesyciLvlSetWaiting();
		else
			if (scanName ~= name or scanRealm ~= realm) then
				NotifyInspect("mouseover");
				ki_Event:RegisterEvent("INSPECT_READY");
			else
				KhesyciLvlAddTooltip();
			end
		end
	end
end

function KhesyciLvlInspect(guid)
	if UnitIsPlayer("mouseover") then
		local newName, newRealm = UnitName("mouseover");
		if (newRealm == nil or newRealm == '') then
			newRealm = GetRealmName();
		end

		if (newName == scanName and newRealm == scanRealm) then
			name = newName;
			realm = newRealm;

			_, _, class = UnitClass("mouseover");
			_, spec = GetSpecializationInfoByID(GetInspectSpecialization("mouseover"));
			iLvl = C_PaperDollInfo.GetInspectItemLevel("mouseover");

			KhesyciLvlAddTooltip();
		end
	end
end

function KhesyciLvlAddTooltip()
	local addLine = true;
	local r, g, b = KhesyciLvlGetClassColor(class);

	for i = 1, GameTooltip:NumLines() do
		local line = _G["GameTooltipTextLeft"..i]:GetText()
		if (line == spec or line == "iLvl: " .. iLvl) then
			addLine = false;
		end
	end

	if (addLine and iLvl > 0) then
		if (ki_addLine) then
			GameTooltip:AddLine(" ");
		end
		GameTooltip:AddLine(spec, r, g, b, 1);
		GameTooltip:AddLine("iLvl: " .. iLvl);
		GameTooltip:Show();
	end
end